@@ -29,6 +29,7 @@
 #include "fs/Path.hxx"
 #include "system/Error.hxx"
 #include "util/RuntimeError.hxx"
+#include "util/UTF8.hxx"
 
 #include <zzip/zzip.h>
 
@@ -84,7 +85,7 @@ ZzipArchiveFile::Visit(ArchiveVisitor &visitor)
 	ZZIP_DIRENT dirent;
 	while (zzip_dir_read(dir->dir, &dirent))
 		//add only files
-		if (dirent.st_size > 0)
+		if (dirent.st_size > 0 && ValidateUTF8(dirent.d_name))
 			visitor.VisitArchiveEntry(dirent.d_name);
 }
 
@@ -116,6 +117,9 @@ public:
 		zzip_file_close(file);
 	}
 
+	ZzipInputStream(const ZzipInputStream &) = delete;
+	ZzipInputStream &operator=(const ZzipInputStream &) = delete;
+
 	/* virtual methods from InputStream */
 	[[nodiscard]] bool IsEOF() const noexcept override;
 	size_t Read(std::unique_lock<Mutex> &lock,
@@ -185,7 +189,7 @@ ZzipInputStream::Seek(std::unique_lock<Mutex> &, offset_type new_offset)
 
 /* exported structures */
 
-static const char *const zzip_archive_extensions[] = {
+static constexpr const char *zzip_archive_extensions[] = {
 	"zip",
 	nullptr
 };
