@@ -22,6 +22,7 @@
 #include "PlaylistFile.hxx"
 #include "Interface.hxx"
 #include "song/DetachedSong.hxx"
+#include "protocol/Ack.hxx"
 
 #include <functional>
 
@@ -41,3 +42,42 @@ search_add_to_playlist(const Database &db, const Storage *storage,
 	const auto f = [=](auto && arg1) { return AddSong(storage, playlist_path_utf8, arg1); };
 	db.Visit(selection, f);
 }
+
+unsigned
+SearchInsertIntoPlaylist(const Database &db, const Storage *storage,
+			 const DatabaseSelection &selection,
+			 PlaylistFileEditor &playlist,
+			 unsigned position)
+{
+	assert(position <= playlist.size());
+
+	unsigned n = 0;
+
+	db.Visit(selection, [&playlist, &position, &n, storage](const auto &song){
+		playlist.Insert(position + n,
+				DatabaseDetachSong(storage, song));
+		++position;
+		++n;
+	});
+
+	return n;
+}
+
+void
+SearchInsertIntoPlaylist(const Database &db, const Storage *storage,
+			 const DatabaseSelection &selection,
+			 const char *playlist_name,
+			 unsigned position)
+{
+	PlaylistFileEditor editor{
+		playlist_name,
+		PlaylistFileEditor::LoadMode::TRY,
+	};
+
+	if (position > editor.size())
+		throw ProtocolError{ACK_ERROR_ARG, "Bad position"};
+
+	if (SearchInsertIntoPlaylist(db, storage, selection,
+				     editor, position) > 0)
+		editor.Save();
+}
