@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2019 Max Kellermann <max.kellermann@gmail.com>
+ * Copyright 2008-2020 Max Kellermann <max.kellermann@gmail.com>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,25 +29,24 @@
 
 #include "Global.hxx"
 #include "Request.hxx"
-#include "Log.hxx"
 #include "event/Loop.hxx"
-#include "event/SocketMonitor.hxx"
-#include "util/RuntimeError.hxx"
-#include "util/Domain.hxx"
+#include "event/SocketEvent.hxx"
+#include "util/Compiler.h"
 
 #include <cassert>
 
-static constexpr Domain curlm_domain("curlm");
-
 /**
  * Monitor for one socket created by CURL.
  */
-class CurlSocket final : SocketMonitor {
+class CurlSocket final {
 	CurlGlobal &global;
 
+	SocketEvent socket_event;
+
 public:
 	CurlSocket(CurlGlobal &_global, EventLoop &_loop, SocketDescriptor _fd)
-		:SocketMonitor(_fd, _loop), global(_global) {}
+		:global(_global),
+		 socket_event(_loop, BIND_THIS_METHOD(OnSocketReady), _fd) {}
 
 	~CurlSocket() noexcept {
 		/* TODO: sometimes, CURL uses CURL_POLL_REMOVE after
@@ -59,6 +58,13 @@ public:
 		   better solution? */
 	}
 
+	CurlSocket(const CurlSocket &) = delete;
+	CurlSocket &operator=(const CurlSocket &) = delete;
+
+	auto &GetEventLoop() const noexcept {
+		return socket_event.GetEventLoop();
+	}
+
 	/**
 	 * Callback function for CURLMOPT_SOCKETFUNCTION.
 	 */
@@ -66,29 +72,33 @@ public:
 				  curl_socket_t s, int action,
 				  void *userp, void *socketp) noexcept;
 
-	bool OnSocketReady(unsigned flags) noexcept override;
-
 private:
-	static constexpr int FlagsToCurlCSelect(unsigned flags) noexcept {
-		return (flags & (READ | HANGUP) ? CURL_CSELECT_IN : 0) |
-			(flags & WRITE ? CURL_CSELECT_OUT : 0) |
-			(flags & ERROR ? CURL_CSELECT_ERR : 0);
+	SocketDescriptor GetSocket() const noexcept {
+		return socket_event.GetSocket();
 	}
 
-	gcc_const
+	void OnSocketReady(unsigned events) noexcept;
+
+	static constexpr int FlagsToCurlCSelect(unsigned flags) noexcept {
+		return (flags & (SocketEvent::READ | SocketEvent::HANGUP) ? CURL_CSELECT_IN : 0) |
+			(flags & SocketEvent::WRITE ? CURL_CSELECT_OUT : 0) |
+			(flags & SocketEvent::ERROR ? CURL_CSELECT_ERR : 0);
+	}
+
+	[[gnu::const]]
 	static unsigned CurlPollToFlags(int action) noexcept {
 		switch (action) {
 		case CURL_POLL_NONE:
 			return 0;
 
 		case CURL_POLL_IN:
-			return READ;
+			return SocketEvent::READ;
 
 		case CURL_POLL_OUT:
-			return WRITE;
+			return SocketEvent::WRITE;
 
 		case CURL_POLL_INOUT:
-			return READ|WRITE;
+			return SocketEvent::READ|SocketEvent::WRITE;
 		}
 
 		assert(false);
@@ -130,17 +140,16 @@ CurlSocket::SocketFunction([[maybe_unused]] CURL *easy,
 
 	unsigned flags = CurlPollToFlags(action);
 	if (flags != 0)
-		cs->Schedule(flags);
+		cs->socket_event.Schedule(flags);
 	return 0;
 }
 
-bool
+void
 CurlSocket::OnSocketReady(unsigned flags) noexcept
 {
 	assert(GetEventLoop().IsInside());
 
 	global.SocketAction(GetSocket().Get(), FlagsToCurlCSelect(flags));
-	return true;
 }
 
 void
@@ -148,10 +157,7 @@ CurlGlobal::Add(CurlRequest &r)
 {
 	assert(GetEventLoop().IsInside());
 
-	CURLMcode mcode = curl_multi_add_handle(multi.Get(), r.Get());
-	if (mcode != CURLM_OK)
-		throw FormatRuntimeError("curl_multi_add_handle() failed: %s",
-					 curl_multi_strerror(mcode));
+	multi.Add(r.Get());
 
 	InvalidateSockets();
 }
@@ -161,13 +167,13 @@ CurlGlobal::Remove(CurlRequest &r) noexcept
 {
 	assert(GetEventLoop().IsInside());
 
-	curl_multi_remove_handle(multi.Get(), r.Get());
+	multi.Remove(r.Get());
 }
 
 /**
  * Find a request by its CURL "easy" handle.
  */
-gcc_pure
+[[gnu::pure]]
 static CurlRequest *
 ToRequest(CURL *easy) noexcept
 {
@@ -185,10 +191,8 @@ CurlGlobal::ReadInfo() noexcept
 	assert(GetEventLoop().IsInside());
 
 	CURLMsg *msg;
-	int msgs_in_queue;
 
-	while ((msg = curl_multi_info_read(multi.Get(),
-					   &msgs_in_queue)) != nullptr) {
+	while ((msg = multi.InfoRead()) != nullptr) {
 		if (msg->msg == CURLMSG_DONE) {
 			auto *request = ToRequest(msg->easy_handle);
 			if (request != nullptr)
@@ -203,10 +207,7 @@ CurlGlobal::SocketAction(curl_socket_t fd, int ev_bitmask) noexcept
 	int running_handles;
 	CURLMcode mcode = curl_multi_socket_action(multi.Get(), fd, ev_bitmask,
 						   &running_handles);
-	if (mcode != CURLM_OK)
-		FormatError(curlm_domain,
-			    "curl_multi_socket_action() failed: %s",
-			    curl_multi_strerror(mcode));
+	(void)mcode;
 
 	defer_read_info.Schedule();
 }
