@@ -33,23 +33,27 @@
 static constexpr PeriodClock::Duration REOPEN_AFTER = std::chrono::seconds(10);
 
 AudioOutputControl::AudioOutputControl(std::unique_ptr<FilteredAudioOutput> _output,
-				       AudioOutputClient &_client) noexcept
+				       AudioOutputClient &_client,
+				       const ConfigBlock &block)
 	:output(std::move(_output)),
 	 name(output->GetName()),
 	 client(_client),
-	 thread(BIND_THIS_METHOD(Task))
+	 thread(BIND_THIS_METHOD(Task)),
+	 tags(block.GetBlockValue("tags", true)),
+	 always_on(block.GetBlockValue("always_on", false)),
+	 enabled(block.GetBlockValue("enabled", true))
 {
 }
 
-AudioOutputControl::AudioOutputControl(AudioOutputControl *_output,
+AudioOutputControl::AudioOutputControl(AudioOutputControl &&src,
 				       AudioOutputClient &_client) noexcept
-	:output(_output->Steal()),
+	:output(src.Steal()),
 	 name(output->GetName()),
 	 client(_client),
-	 thread(BIND_THIS_METHOD(Task))
+	 thread(BIND_THIS_METHOD(Task)),
+	 tags(src.tags),
+	 always_on(src.always_on)
 {
-     tags =_output->tags;
-	 always_on=_output->always_on;
 }
 
 AudioOutputControl::~AudioOutputControl() noexcept
@@ -57,14 +61,6 @@ AudioOutputControl::~AudioOutputControl() noexcept
 	StopThread();
 }
 
-void
-AudioOutputControl::Configure(const ConfigBlock &block)
-{
-	tags = block.GetBlockValue("tags", true);
-	always_on = block.GetBlockValue("always_on", false);
-	enabled = block.GetBlockValue("enabled", true);
-}
-
 std::unique_ptr<FilteredAudioOutput>
 AudioOutputControl::Steal() noexcept
 {
