@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2020 The Music Player Daemon Project
+ * Copyright 2003-2021 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -53,8 +53,16 @@ Tag::Tag(const Tag &other) noexcept
 	}
 }
 
-std::unique_ptr<Tag>
+Tag
 Tag::Merge(const Tag &base, const Tag &add) noexcept
+{
+	TagBuilder builder(add);
+	builder.Complement(base);
+	return builder.Commit();
+}
+
+std::unique_ptr<Tag>
+Tag::MergePtr(const Tag &base, const Tag &add) noexcept
 {
 	TagBuilder builder(add);
 	builder.Complement(base);
@@ -70,7 +78,23 @@ Tag::Merge(std::unique_ptr<Tag> base, std::unique_ptr<Tag> add) noexcept
 	if (base == nullptr)
 		return add;
 
-	return Merge(*base, *add);
+	return MergePtr(*base, *add);
+}
+
+std::unique_ptr<Tag>
+Tag::Merge(const Tag *base, const Tag *add) noexcept
+{
+	if (base == nullptr && add == nullptr)
+		/* no tag */
+		return nullptr;
+
+	if (base == nullptr)
+		return std::make_unique<Tag>(*add);
+
+	if (add == nullptr)
+		return std::make_unique<Tag>(*base);
+
+	return MergePtr(*base, *add);
 }
 
 const char *
